@@ -38,10 +38,12 @@
 
 #ifdef NM_LIBRESWAN_OLD
 #define NM_VPN_LIBNM_COMPAT
+
 #include <nm-vpn-plugin-ui-interface.h>
 #include <nm-setting-vpn.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-ip4-config.h>
+#include <nm-ui-utils.h>
 
 #define LIBRESWAN_EDITOR_PLUGIN_ERROR                  NM_SETTING_VPN_ERROR
 #define LIBRESWAN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY NM_SETTING_VPN_ERROR_INVALID_PROPERTY
@@ -49,6 +51,7 @@
 #else /* !NM_LIBRESWAN_OLD */
 
 #include <NetworkManager.h>
+#include <nma-ui-utils.h>
 
 #define LIBRESWAN_EDITOR_PLUGIN_ERROR                  NM_CONNECTION_ERROR
 #define LIBRESWAN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY NM_CONNECTION_ERROR_INVALID_PROPERTY
@@ -59,7 +62,7 @@
 #include "utils.h"
 
 #define LIBRESWAN_PLUGIN_NAME    _("IPsec based VPN")
-#define LIBRESWAN_PLUGIN_DESC    _("IPsec, IKEv1, IKEv2 based VPN")
+#define LIBRESWAN_PLUGIN_DESC    _("IPsec based VPN using IKEv1")
 
 #define ENC_TYPE_SECURE 0
 #define ENC_TYPE_WEAK   1
@@ -119,16 +122,6 @@ check_validity (LibreswanEditor *self, GError **error)
 		return FALSE;
 	}
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_entry"));
-	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
-	if (!str || !strlen (str)) {
-		g_set_error (error,
-		             LIBRESWAN_EDITOR_PLUGIN_ERROR,
-		             LIBRESWAN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
-		             NM_LIBRESWAN_LEFTID);
-		return FALSE;
-	}
-
 	return TRUE;
 }
 
@@ -146,13 +139,9 @@ setup_password_widget (LibreswanEditor *self,
                        gboolean new_connection)
 {
 	LibreswanEditorPrivate *priv = LIBRESWAN_EDITOR_GET_PRIVATE (self);
-	NMSettingSecretFlags secret_flags = NM_SETTING_SECRET_FLAG_NONE;
 	GtkWidget *widget;
 	const char *value;
 
-	if (new_connection)
-		secret_flags = NM_SETTING_SECRET_FLAG_AGENT_OWNED;
-
 	widget = (GtkWidget *) gtk_builder_get_object (priv->builder, entry_name);
 	g_assert (widget);
 	gtk_size_group_add_widget (priv->group, widget);
@@ -160,10 +149,7 @@ setup_password_widget (LibreswanEditor *self,
 	if (s_vpn) {
 		value = nm_setting_vpn_get_secret (s_vpn, secret_name);
 		gtk_entry_set_text (GTK_ENTRY (widget), value ? value : "");
-		nm_setting_get_secret_flags (NM_SETTING (s_vpn), secret_name, &secret_flags, NULL);
 	}
-	secret_flags &= ~(NM_SETTING_SECRET_FLAG_NOT_SAVED | NM_SETTING_SECRET_FLAG_NOT_REQUIRED);
-	g_object_set_data (G_OBJECT (widget), "flags", GUINT_TO_POINTER (secret_flags));
 
 	g_signal_connect (widget, "changed", G_CALLBACK (stuff_changed_cb), self);
 }
@@ -187,48 +173,13 @@ show_toggled_cb (GtkCheckButton *button, LibreswanEditor *self)
 }
 
 static void
-pw_type_changed_helper (LibreswanEditor *self, GtkWidget *combo)
-{
-	LibreswanEditorPrivate *priv = LIBRESWAN_EDITOR_GET_PRIVATE (self);
-	const char *entry = NULL;
-	GtkWidget *widget;
-
-	/* If the user chose "Not required", desensitize and clear the correct
-	 * password entry.
-	 */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_pass_type_combo"));
-	if (combo == widget)
-		entry = "user_password_entry";
-	else {
-		widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_pass_type_combo"));
-		if (combo == widget)
-			entry = "group_password_entry";
-	}
-	if (!entry)
-		return;
-
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, entry));
-	g_assert (widget);
-
-	switch (gtk_combo_box_get_active (GTK_COMBO_BOX (combo))) {
-	case PW_TYPE_ASK:
-	case PW_TYPE_UNUSED:
-		gtk_entry_set_text (GTK_ENTRY (widget), "");
-		gtk_widget_set_sensitive (widget, FALSE);
-		break;
-	default:
-		gtk_widget_set_sensitive (widget, TRUE);
-		break;
-	}
-}
-
-static void
-pw_type_combo_changed_cb (GtkWidget *combo, gpointer user_data)
+password_storage_changed_cb (GObject *entry,
+			     GParamSpec *pspec,
+			     gpointer user_data)
 {
 	LibreswanEditor *self = LIBRESWAN_EDITOR (user_data);
 
-	pw_type_changed_helper (self, combo);
-	stuff_changed_cb (combo, self);
+	stuff_changed_cb (NULL, self);
 }
 
 static const char *
@@ -247,77 +198,45 @@ secret_flags_to_pw_type (NMSettingVpn *s_vpn, const char *key)
 }
 
 static void
-init_one_pw_combo (LibreswanEditor *self,
-                   NMSettingVpn *s_vpn,
-                   const char *combo_name,
-                   const char *secret_key,
-                   const char *type_key,
-                   const char *entry_name)
+init_password_icon (LibreswanEditor *self,
+                    NMSettingVpn *s_vpn,
+                    const char *secret_key,
+                    const char *type_key,
+                    const char *entry_name)
 {
 	LibreswanEditorPrivate *priv = LIBRESWAN_EDITOR_GET_PRIVATE (self);
-	int active = -1;
-	GtkWidget *widget;
-	GtkListStore *store;
-	GtkTreeIter iter;
-	const char *value = NULL;
-	guint32 default_idx = 1;
+	GtkWidget *entry;
+	const char *value;
+	const char *flags = NULL;
 
-	/* If there's already a password and the password type can't be found in
-	 * the VPN settings, default to saving it.  Otherwise, always ask for it.
+	entry = GTK_WIDGET (gtk_builder_get_object (priv->builder, entry_name));
+	g_assert (entry);
+
+	nma_utils_setup_password_storage (entry, 0, (NMSetting *) s_vpn, secret_key,
+	                                   TRUE, FALSE);
+
+	/* If there's no password and no flags in the setting,
+	 * initialize flags as "always-ask".
 	 */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, entry_name));
-	if (widget) {
-		const char *tmp;
-
-		tmp = gtk_entry_get_text (GTK_ENTRY (widget));
-		if (tmp && strlen (tmp))
-			default_idx = 0;
-	}
-
-	store = gtk_list_store_new (1, G_TYPE_STRING);
 	if (s_vpn) {
-		value = secret_flags_to_pw_type (s_vpn, secret_key);
-		if (!value)
-			value = nm_setting_vpn_get_data_item (s_vpn, type_key);
+		flags = secret_flags_to_pw_type (s_vpn, secret_key);
+		if (!flags || !strcmp (flags, NM_LIBRESWAN_PW_TYPE_SAVE))
+			flags = nm_setting_vpn_get_data_item (s_vpn, type_key);
 	}
+	value = gtk_entry_get_text (GTK_ENTRY (entry));
+	if ((!value || !*value) && !flags)
+		nma_utils_update_password_storage (entry, NM_SETTING_SECRET_FLAG_NOT_SAVED,
+	                                           (NMSetting *) s_vpn, secret_key);
 
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("Saved"), -1);
-	if ((active < 0) && value) {
-		if (!strcmp (value, NM_LIBRESWAN_PW_TYPE_SAVE))
-			active = 0;
-	}
-
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("Always Ask"), -1);
-	if ((active < 0) && value) {
-		if (!strcmp (value, NM_LIBRESWAN_PW_TYPE_ASK))
-			active = 1;
-	}
-
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("Not Required"), -1);
-	if ((active < 0) && value) {
-		if (!strcmp (value, NM_LIBRESWAN_PW_TYPE_UNUSED))
-			active = 2;
-	}
-
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, combo_name));
-	g_assert (widget);
-	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (store));
-	g_object_unref (store);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? default_idx : active);
-	pw_type_changed_helper (self, widget);
-
-	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (pw_type_combo_changed_cb), self);
+	g_signal_connect (entry, "notify::secondary-icon-name",
+	                          G_CALLBACK (password_storage_changed_cb), self);
 }
 
-
 static gboolean
 init_editor_plugin (LibreswanEditor *self,
-                NMConnection *connection,
-                gboolean new_connection,
-                GError **error)
+                    NMConnection *connection,
+                    gboolean new_connection,
+                    GError **error)
 {
 	LibreswanEditorPrivate *priv = LIBRESWAN_EDITOR_GET_PRIVATE (self);
 	NMSettingVpn *s_vpn = NULL;
@@ -362,18 +281,16 @@ init_editor_plugin (LibreswanEditor *self,
 	                       NM_LIBRESWAN_PSK_VALUE,
 	                       new_connection);
 
-	init_one_pw_combo (self,
-	                   s_vpn,
-	                   "user_pass_type_combo",
-	                   NM_LIBRESWAN_XAUTH_PASSWORD,
-	                   NM_LIBRESWAN_XAUTH_PASSWORD_INPUT_MODES,
-	                   "user_password_entry");
-	init_one_pw_combo (self,
-	                   s_vpn,
-	                   "group_pass_type_combo",
-	                   NM_LIBRESWAN_PSK_VALUE,
-	                   NM_LIBRESWAN_PSK_INPUT_MODES,
-	                   "group_password_entry");
+	init_password_icon (self,
+	                    s_vpn,
+	                    NM_LIBRESWAN_XAUTH_PASSWORD,
+	                    NM_LIBRESWAN_XAUTH_PASSWORD_INPUT_MODES,
+	                    "user_password_entry");
+	init_password_icon (self,
+	                    s_vpn,
+	                    NM_LIBRESWAN_PSK_VALUE,
+	                    NM_LIBRESWAN_PSK_INPUT_MODES,
+	                    "group_password_entry");
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_entry"));
 	g_return_val_if_fail (widget != NULL, FALSE);
@@ -444,33 +361,32 @@ static void
 save_one_password (NMSettingVpn *s_vpn,
                    GtkBuilder *builder,
                    const char *entry_name,
-                   const char *combo_name,
                    const char *secret_key,
                    const char *type_key)
 {
-	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
+	NMSettingSecretFlags flags;
 	const char *data_val = NULL, *password;
-	GtkWidget *entry, *combo;
+	GtkWidget *entry;
 
+	/* Get secret flags */
 	entry = GTK_WIDGET (gtk_builder_get_object (builder, entry_name));
-	flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (entry), "flags"));
+	flags = nma_utils_menu_to_secret_flags (entry);
 
-	combo = GTK_WIDGET (gtk_builder_get_object (builder, combo_name));
-	switch (gtk_combo_box_get_active (GTK_COMBO_BOX (combo))) {
-	case PW_TYPE_SAVE:
+	/* Save password and convert flags to legacy data items */
+	switch (flags) {
+	case NM_SETTING_SECRET_FLAG_NONE:
+	case NM_SETTING_SECRET_FLAG_AGENT_OWNED:
 		password = gtk_entry_get_text (GTK_ENTRY (entry));
 		if (password && strlen (password))
 			nm_setting_vpn_add_secret (s_vpn, secret_key, password);
 		data_val = NM_LIBRESWAN_PW_TYPE_SAVE;
 		break;
-	case PW_TYPE_UNUSED:
+	case NM_SETTING_SECRET_FLAG_NOT_REQUIRED:
 		data_val = NM_LIBRESWAN_PW_TYPE_UNUSED;
-		flags |= NM_SETTING_SECRET_FLAG_NOT_REQUIRED;
 		break;
-	case PW_TYPE_ASK:
+	case NM_SETTING_SECRET_FLAG_NOT_SAVED:
 	default:
 		data_val = NM_LIBRESWAN_PW_TYPE_ASK;
-		flags |= NM_SETTING_SECRET_FLAG_NOT_SAVED;
 		break;
 	}
 
@@ -535,13 +451,11 @@ update_connection (NMVpnEditor *iface,
 	save_one_password (s_vpn,
 	                   priv->builder,
 	                   "user_password_entry",
-	                   "user_pass_type_combo",
 	                   NM_LIBRESWAN_XAUTH_PASSWORD,
 	                   NM_LIBRESWAN_XAUTH_PASSWORD_INPUT_MODES);
 	save_one_password (s_vpn,
 	                   priv->builder,
 	                   "group_password_entry",
-	                   "group_pass_type_combo",
 	                   NM_LIBRESWAN_PSK_VALUE,
 	                   NM_LIBRESWAN_PSK_INPUT_MODES);
 
@@ -559,7 +473,7 @@ is_new_func (const char *key, const char *value, gpointer user_data)
 }
 
 static NMVpnEditor *
-nm_vpn_editor_interface_new (NMConnection *connection, GError **error)
+nm_vpn_editor_new (NMConnection *connection, GError **error)
 {
 	NMVpnEditor *object;
 	LibreswanEditorPrivate *priv;
@@ -622,6 +536,17 @@ dispose (GObject *object)
 	LibreswanEditor *plugin = LIBRESWAN_EDITOR (object);
 	LibreswanEditorPrivate *priv = LIBRESWAN_EDITOR_GET_PRIVATE (plugin);
 
+	GtkWidget *widget;
+
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_password_entry"));
+	g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
+	                                      (GCallback) password_storage_changed_cb,
+	                                      plugin);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_password_entry"));
+	g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
+	                                      (GCallback) password_storage_changed_cb,
+	                                      plugin);
+
 	if (priv->group)
 		g_object_unref (priv->group);
 
@@ -712,7 +637,7 @@ get_capabilities (NMVpnEditorPlugin *iface)
 static NMVpnEditor *
 get_editor (NMVpnEditorPlugin *iface, NMConnection *connection, GError **error)
 {
-	return nm_vpn_editor_interface_new (connection, error);
+	return nm_vpn_editor_new (connection, error);
 }
 
 static void
